#!/usr/bin/python

import RPi.GPIO as GPIO
import time, random, signal, os, math, lcddriver, logger, atexit
from MCP3008 import MCP3008
from configobj import ConfigObj


#############################################################################
# Globale Variablen #########################################################
#############################################################################

# Konfiguration ############
pin_close = 6
pin_stop = 26
pin_open = 19
pin_menu = 13
pin_open_sens = 16
pin_close_sens = 20
pin_motor_open = 22
pin_motor_close = 27
pin_motor_pwr = 17
pin_heat = 24
pin_led = 23
pin_lcd_led = 4
ad_light = 7
ad_temp = 3
motor_timeout = 60
auto_open=2
auto_open_h=6
auto_open_m=0
auto_open_lt=300
auto_close=2
auto_close_h=21
auto_close_m=0
auto_close_lt=100
auto_heat=1
auto_heat_tt=350
auto_heat_tc=200
sleeptime=1
lcd_led_time=30
lcd_reset=60
led_steps = []
auto_mlight=1
auto_mlight_h=5
auto_mlight_m=0
auto_mlight_max=127
auto_mlight_ft=30
auto_mlight_lt=290
auto_lc=60
auto_alight=1
auto_alight_lt=300
auto_alight_h=19
auto_alight_m=0
max_menu=19

# Arbeitsvariablen
started = 0
motor_status = 0
menu_mode = 0
auto_open_status = 0
auto_close_status = 0
auto_open_lcc=0
auto_close_lcc=0
auto_close_lccr=0
auto_heat_tcc=0
auto_heat_tccr=0
auto_heat_status=0
auto_heat_toff=0
lcd_led_cur=-1
lcd_reset_cur=lcd_reset
auto_mlight_pwm=0
auto_mlight_sp=0
auto_mlight_status=0
auto_mlight_lcc=0
auto_mlight_lccr=0
auto_alight_status=0
auto_alight_lcc=0
auto_alight_lccr=0
tmp_h=0
tmp_m=0
cur_page=0
clock_ch=False
man_heater=False
man_light=False

#############################################################################
# Konfigurationsdateien einlesen und Logger konfigurieren ###################
#############################################################################

#PWM-Tabelle fuer LEDs ######################################################
pwmf = open("/home/pi/dev/python/chickensitter/led_steps.conf","r")
for line in pwmf:
   led_steps.append(float(line))
pwmf.close()

# Sensor logger kofigurieren/aktivieren #####################################
senslogger = logger.Sens("/home/pi/dev/python/chickensitter/sensors.log",5)
senslogger.enable()

# Action loffer konfigurieren ###############################################
actionlogger = logger.ActionLog("/home/pi/dev/python/chickensitter/chickensitter.log")

# Hauptconfig objekt erstellen und config lesen #############################
config = ConfigObj("/home/pi/dev/python/chickensitter/chickensitter.conf")
if os.path.isfile(config.filename):
   auto_open = int(config["auto_open"])
   auto_open_h = int(config['auto_open_h'])
   auto_open_m = int(config['auto_open_m'])
   auto_open_lt = int(config['auto_open_lt'])
   auto_close = int(config['auto_close'])
   auto_close_h = int(config['auto_close_h'])
   auto_close_m = int(config['auto_close_m'])
   auto_close_lt = int(config['auto_close_lt'])
   auto_heat = int(config['auto_heat'])
   auto_heat_tt = int(config['auto_heat_tt'])
   auto_mlight = int(config['auto_mlight'])
   auto_mlight_h = int(config['auto_mlight_h'])
   auto_mlight_m = int(config['auto_mlight_m'])
   auto_mlight_max = int(config['auto_mlight_max'])
   auto_mlight_ft = int(config['auto_mlight_ft'])
   auto_mlight_lt = int(config['auto_mlight_lt'])
   auto_lc = int(config['auto_lc'])
   motor_timeout = int(config['motor_timeout'])
   auto_alight = int(config['auto_alight'])
   auto_alight_h = int(config['auto_alight_h'])
   auto_alight_m = int(config['auto_alight_m'])
   auto_alight_lt = int(config['auto_alight_lt'])

def write_config():
   global config
   global auto_open, auto_open_h, auto_open_m, auto_open_lt
   global auto_close, auto_close_h, auto_close_m, auto_close_lt
   global auto_heat, auto_heat_tt
   global auto_mlight, auto_mlight_h, auto_mlight_m, auto_mlight_max, auto_mlight_ft, auto_mlight_lt
   global auto_lc, motor_timeout
   config['auto_open'] = auto_open
   config['auto_open_h'] = auto_open_h
   config['auto_open_m'] = auto_open_m
   config['auto_open_lt'] = auto_open_lt
   config['auto_close'] = auto_close
   config['auto_close_h'] = auto_close_h
   config['auto_close_m'] = auto_close_m
   config['auto_close_lt'] = auto_close_lt
   config['auto_heat'] = auto_heat
   config['auto_heat_tt'] = auto_heat_tt
   config['auto_mlight'] = auto_mlight
   config['auto_mlight_h'] = auto_mlight_h
   config['auto_mlight_m'] = auto_mlight_m
   config['auto_mlight_max'] = auto_mlight_max
   config['auto_mlight_ft'] = auto_mlight_ft
   config['auto_mlight_lt'] = auto_mlight_lt
   config['auto_lc'] = auto_lc
   config['motor_timeout'] = motor_timeout
   config['auto_alight'] = auto_alight
   config['auto_alight_h'] = auto_alight_h
   config['auto_alight_m'] = auto_alight_m
   config['auto_alight_lt'] = auto_alight_lt
   config.write()


#############################################################################
# GPIO Setup ################################################################
#############################################################################
GPIO.setmode(GPIO.BCM)
GPIO.setup(pin_close,GPIO.IN)
GPIO.setup(pin_stop,GPIO.IN)
GPIO.setup(pin_open,GPIO.IN)
GPIO.setup(pin_menu,GPIO.IN)
GPIO.setup(pin_open_sens,GPIO.IN)
GPIO.setup(pin_close_sens,GPIO.IN)
GPIO.setup(pin_motor_open,GPIO.OUT,initial=0)
GPIO.setup(pin_motor_close,GPIO.OUT,initial=0)
GPIO.setup(pin_motor_pwr,GPIO.OUT,initial=0)
GPIO.setup(pin_heat,GPIO.OUT,initial=0)
GPIO.setup(pin_led,GPIO.OUT,initial=0)
GPIO.setup(pin_lcd_led,GPIO.OUT,initial=0)


###############################################################################
# Externe Geraete konfigurieren ###############################################
###############################################################################

# A/D-Wandler #############################
adc = MCP3008()

# LCD Display #############################
lcd = lcddriver.lcd()
lcd.lcd_backlight("ON")
lcd.lcd_clear()
lcd.lcd_display_string("Chickensitter",1)
lcd.lcd_display_string("startet...",2)
GPIO.output(pin_lcd_led,1)
lcd_led_cur=lcd_led_time

# LED PWM-Engine #########################
led = GPIO.PWM(pin_led,100)



############################################################################
# Funktion setzt die Systemzeit ############################################
############################################################################
def set_systemtime(h,m):
   os.system("sudo /bin/date --set="+str(h).zfill(2)+":"+str(m).zfill(2)+":00")
   os.system("sudo hwclock -w")


############################################################################
# Funktion gibt aktuelle Informationen bzw. Menu am Display aus ############
############################################################################
def draw_lcd(akt_lv,akt_tv):
 
   global lcd
   global lcd_reset_cur

   if lcd_reset_cur == 0:
      lcd = lcddriver.lcd()
      lcd.lcd_clear()
      lcd_reset_cur=lcd_reset

   # Tuere in Bewegung ####################################################
   if menu_mode == -1:
     row_1 = "In Transit..."
     row_2 = ""
     
   # Menu nicht geoeffnet - Status ausgeben ###############################
   elif menu_mode == 0:
      row_1 = time.strftime("%H:%M")+" L"+str(akt_lv)+"T"+str(akt_tv)
      if cur_page == 0: row_2="[D:"
      else: row_2="D:"
      if get_status() == 1:
         row_2=row_2+"C"
      elif get_status() == -1:
         row_2=row_2+"O"
      else:
         row_2=row_2+"?"
      if cur_page == 0: row_2=row_2+"] "
      else: row_2=row_2+" "
      
      if cur_page == 1: row_2=row_2+"["
      row_2=row_2+"H:"+str(auto_heat_status)+" L:"
      if auto_mlight_status > 0 and auto_mlight_status < 4 or man_light==True: row_2=row_2+"1"
      else: row_2=row_2+"0"
      if cur_page == 1: row_2=row_2+"]"

   # Erste Menu-Seite - Uhrzeit ##########################################
   elif menu_mode == 1:
      row_1 = "Uhrzeit"
      row_2 = str(tmp_h).zfill(2)+":"+str(tmp_m).zfill(2)
   
   # Zweite Menu-Seite - Modus fuer Oeffnen ##############################
   elif menu_mode == 2:
      row_1 ="Oeffnen Modus"
      if auto_open == 0:
         row_2="Manuell"
      elif auto_open == 1:
         row_2="Automatisch"
      elif auto_open == 2:
         row_2="Auto mit Licht"

   # Dritte Menu-Seite - Zeit fuer Oeffnen ###############################
   elif menu_mode == 3:
      row_1="Oeffnen Zeit"
      row_2=str(auto_open_h).zfill(2)+":"+str(auto_open_m).zfill(2)

   # vierte Menu-Seite - Lichtwert fuer Oeffnen ##########################
   elif menu_mode == 4:
      row_1="Oeffnen LWert"
      row_2=str(auto_open_lt)

   # Fuenfte Menu-Seite - Modus fuer Schliessen ##############################
   elif menu_mode == 5:
      row_1 ="Schliesen Modus"
      if auto_close == 0:
         row_2="Manuell"
      elif auto_close == 1:
         row_2="Automatisch"
      elif auto_close == 2:
         row_2="Auto mit Licht"

   # Sechste Menu-Seite - Zeit fuer Schliessen ###############################
   elif menu_mode == 6:
      row_1="Schliessen Zeit"
      row_2=str(auto_close_h).zfill(2)+":"+str(auto_close_m).zfill(2)

   # Siebente Menu-Seite - Lichtwert fuer Schliessen ##########################
   elif menu_mode == 7:
      row_1="Schliessen LWert"
      row_2=str(auto_close_lt)

   # Achte Menu-Seite - Modus fuer Heizer ##############################
   elif menu_mode == 8:
      row_1 ="Heizer Modus"
      if auto_heat == 0:
         row_2="Manuell"
      elif auto_heat == 1:
         row_2="Auto nach Temp"

   # Neunte Menu-Seite - Tempwert fuer Heizer ##########################
   elif menu_mode == 9:
      row_1="Heizer TWert"
      row_2=str(auto_heat_tt)

   # 10te Menu-Seite - Modus fuer Morgenlicht ###########################
   elif menu_mode == 10:
      row_1 ="Morgen Licht"
      if auto_mlight == 0:
         row_2="Manuell"
      elif auto_mlight == 1:
         row_2="Automatisch"

   # 11te Menu-Seite - Zeit fuer Morgenlicht ###############################
   elif menu_mode == 11:
      row_1="M-Licht Ein Zeit"
      row_2=str(auto_mlight_h).zfill(2)+":"+str(auto_mlight_m).zfill(2)

   # 12te Menu-Seite - Lichwert fuer Morgenlicht aus ##########################
   elif menu_mode == 12:
      row_1="M-Licht Aus LVal"
      row_2=str(auto_mlight_lt)

   # 13te Menu-Seite - Modus fuer Abendlicht ###########################
   elif menu_mode == 13:
      row_1 ="Abend Licht"
      if auto_alight == 0:
         row_2="Manuell"
      elif auto_alight == 1:
         row_2="Automatisch"

   # 14te Menu-Seite - Lichwert fuer Abendlicht ein ##########################
   elif menu_mode == 14:
      row_1="A-Licht Ein LVal"
      row_2=str(auto_alight_lt)

   # 15te Menu-Seite - Zeit fuer Abendlicht aus ###############################
   elif menu_mode == 15:
      row_1="M-Licht Aus Zeit"
      row_2=str(auto_alight_h).zfill(2)+":"+str(auto_alight_m).zfill(2)

   # 16te Menu-Seite - Max Morgenlicht Helligkeit ##########################
   elif menu_mode == 16:
      row_1="Licht Max Hell"
      row_2=str(auto_mlight_max)

   # 17te Menu-Seite - Fading Time fuer Licht ######################
   elif menu_mode == 17:
      row_1="Licht Fading"
      row_2=str(auto_mlight_ft)+" Sek"

   # 18te Menu-Seite - Lichtsensor Verzoegerung ######################
   elif menu_mode == 18:
      row_1="LSens Verzoeger."
      row_2=str(auto_lc)+" Sek"

   # 19te Menu-Seite - Motor Timeout ######################
   elif menu_mode == 19:
      row_1="Motor Timeout"
      row_2=str(motor_timeout)+" Sek"
      
   
   # Display Zeilen mit Leerzeichen auffuellen ###########################   
   while len(row_1) < 16:
      row_1 = row_1+" "

   while len(row_2) < 16:
      row_2 = row_2+" "
   
   # Display neu Zeichnen ################################################
   try:
      lcd.lcd_display_string(row_1,1)
      lcd.lcd_display_string(row_2,2)
   except IOError as err:
      actionlogger.log("Display Fehler "+format(err))
      lcd_reset_cur=0



############################################################################
# Funktion gibt Tuerstatus zurueck #########################################
############################################################################
def get_status():
   if GPIO.input(pin_open_sens):
      return -1
   elif GPIO.input(pin_close_sens):
     return 1
   else:
      return 0



############################################################################
# Funktion fuer Motor-Timeout ##############################################
############################################################################
def Timeout_handler(signum, frame):
   if motor_status != 0:
      Set_motor(0)
      actionlogger.log("Timeout: Motor wird gestoppt")
   signal.alarm(0)


############################################################################
# Funktion fuer Motorsteuerung #############################################
############################################################################
def Set_motor(mode):

   global motor_status
   global menu_mode
   global auto_heat_toff

   if mode != 0:
      menu_mode=-1
      draw_lcd(adc.read(channel=ad_light),adc.read(channel=ad_temp))
   else:
      menu_mode=0

   # Antrieb stoppen
   if mode == 0:
      motor_status=0
      GPIO.output(pin_motor_pwr,0)
      signal.alarm(0)
      if auto_heat_toff == 1:
         actionlogger.log("Automatik: Heizer wieder eingeschaltet")
         time.sleep(0.1)
         GPIO.output(pin_heat,1)
         auto_heat_toff=0
  

   # Tuere oeffnen
   elif mode == 1:
      if auto_heat_status == 1:
         actionlogger.log("Automatik: Heizer wird temporaer ausgeschaltet")
         auto_heat_toff=1
         GPIO.output(pin_heat,0)
         time.sleep(0.1)
      motor_status=1
      GPIO.output(pin_motor_open,1)
      time.sleep(0.1)
      GPIO.output(pin_motor_open,0)
      time.sleep(0.1)
      GPIO.output(pin_motor_pwr,1)
      signal.alarm(motor_timeout)

   # Tuere schliessen
   elif mode == -1:
      if auto_heat_status == 1:
         actionlogger.log("Automatik: Heizer wird temporaer ausgeschaltet")
         auto_heat_toff=1
         GPIO.output(pin_heat,0)
         time.sleep(0.1)
      motor_status=-1
      GPIO.output(pin_motor_close,1)
      time.sleep(0.1)
      GPIO.output(pin_motor_close,0)
      time.sleep(0.1)
      GPIO.output(pin_motor_pwr,1)
      signal.alarm(motor_timeout)


############################################################################
# Callback-Funktion fuer Taster ############################################
############################################################################
def Taster_interrupt(channel):

   global lcd_led_cur
   global tmp_h, tmp_m, clock_ch
   global menu_mode, cur_page
   global auto_open, auto_open_h, auto_open_m, auto_open_lt
   global auto_close, auto_close_h, auto_close_m, auto_close_lt
   global auto_heat, auto_heat_tt, auto_heat_status, man_heater
   global auto_mlight, auto_mlight_h, auto_mlight_m, auto_mlight_max, auto_mlight_ft, auto_mlight_lt, auto_mlight_status, man_light, auto_mlight_pwm
   global auto_alight, auto_alight_h, auto_alight_m, auto_alight_lt, auto_alight_status
   global auto_lc, motor_timeout

 
   # Bei Tastenaktivierung LCD Beleuchtung aktivieren
   if GPIO.input(pin_open) or GPIO.input(pin_close) or GPIO.input(pin_stop) or GPIO.input(pin_menu):
      lcd_led_cur=lcd_led_time
      GPIO.output(pin_lcd_led,1)

   
   # Menu Navigation ################################################
   if menu_mode > 0:

      # Menu-Taste -> Menu schliessen
      if channel == pin_menu and GPIO.input(pin_menu):
         if clock_ch:
            set_systemtime(tmp_h,tmp_m)
            clock_ch=False
         write_config()
         menu_mode=0

      # Stop-Taste -> Menuseite wechseln
      elif channel == pin_stop and GPIO.input(pin_stop):
         if menu_mode < max_menu:
            menu_mode=menu_mode+1
         else:
            menu_mode=1

      else:

         # Uhrzeit einstellen
         if menu_mode == 1:
            if channel == pin_close and GPIO.input(pin_close):
               if tmp_h >= 23: tmp_h=0
               else: tmp_h=tmp_h+1
               clock_ch=True
            if channel == pin_open and GPIO.input(pin_open):
               if tmp_m >= 59: tmp_m=0
               else: tmp_m=tmp_m+1    
               clock_ch=True     
	 
         # Oeffnen Modus einstellen
         elif menu_mode == 2:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_open > 0: auto_open=auto_open-1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_open < 2: auto_open=auto_open+1

         # Oeffnen Zeit einstellen
         elif menu_mode == 3:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_open_h >= 23: auto_open_h=0
              else: auto_open_h=auto_open_h+1
           if channel == pin_open and GPIO.input(pin_open):
              if auto_open_m >= 50: auto_open_m=0
              else: auto_open_m=auto_open_m+10

         # Oeffnen Lichtwert einstellen
         elif menu_mode == 4:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_open_lt <= 0: auto_open_lt=1023
              else: auto_open_lt=auto_open_lt-16
           if channel == pin_open and GPIO.input(pin_open):
              if auto_open_lt >= 1023: auto_open_lt=0
              else: auto_open_lt=auto_open_lt+16

         # Schliessen Modus einstellen
         elif menu_mode == 5:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_close > 0: auto_close=auto_close-1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_close < 2: auto_close=auto_close+1

         # Schliessen Zeit einstellen
         elif menu_mode == 6:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_close_h >= 23: auto_close_h=0
              else: auto_close_h=auto_close_h+1
           if channel == pin_open and GPIO.input(pin_open):
              if auto_close_m >= 50: auto_close_m=0
              else: auto_close_m=auto_close_m+10

         # Schliessen Lichtwert einstellen
         elif menu_mode == 7:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_close_lt <= 0: auto_close_lt=1023
              else: auto_close_lt=auto_close_lt-16
           if channel == pin_open and GPIO.input(pin_open):
              if auto_close_lt >= 1023: auto_close_lt=0
              else: auto_close_lt=auto_close_lt+16

         # Heizer Modus einstellen
         elif menu_mode == 8:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_heat > 0: auto_heat=auto_heat-1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_heat < 1: auto_heat=auto_heat+1

         # Heizer Tempwert einstellen
         elif menu_mode == 9:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_heat_tt <= 0: auto_heat_tt=1023
              else: auto_heat_tt=auto_heat_tt-16
           if channel == pin_open and GPIO.input(pin_open):
              if auto_heat_tt >= 1023: auto_heat_tt=0
              else: auto_heat_tt=auto_heat_tt+16

         # Morgen-Licht Modus einstellen
         elif menu_mode == 10:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_mlight > 0: auto_mlight=auto_mlight-1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_mlight < 1: auto_mlight=auto_mlight+1

         # Morgen-Licht Zeit einstellen
         elif menu_mode == 11:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_mlight_h >= 23: auto_mlight_h=0
               else: auto_mlight_h=auto_mlight_h+1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_mlight_m >= 50: auto_mlight_m=0
               else: auto_mlight_m=auto_mlight_m+10

         # Lichtwert fuer Morgenlicht ausschalten einstellen
         elif menu_mode == 12:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_mlight_lt <= 0: auto_mlight_lt=1023
              else: auto_mlight_lt=auto_mlight_lt-16
           if channel == pin_open and GPIO.input(pin_open):
              if auto_mlight_lt >= 1023: auto_mlight_lt=0
              else: auto_mlight_lt=auto_mlight_lt+16

         # Abend-Licht Modus einstellen
         elif menu_mode == 13:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_alight > 0: auto_alight=auto_alight-1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_alight < 1: auto_alight=auto_alight+1

         # Lichtwert fuer Abendlicht einschalten einstellen
         elif menu_mode == 14:
           if channel == pin_close and GPIO.input(pin_close):
              if auto_alight_lt <= 0: auto_alight_lt=1023
              else: auto_alight_lt=auto_alight_lt-16
           if channel == pin_open and GPIO.input(pin_open):
              if auto_alight_lt >= 1023: auto_alight_lt=0
              else: auto_alight_lt=auto_alight_lt+16

         # Abend-Licht Aus-Zeit einstellen
         elif menu_mode == 15:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_alight_h >= 23: auto_alight_h=0
               else: auto_alight_h=auto_alight_h+1
            if channel == pin_open and GPIO.input(pin_open):
               if auto_alight_m >= 50: auto_alight_m=0
               else: auto_alight_m=auto_alight_m+10

         # Licht Max Helligkeit einstellen
         elif menu_mode == 16:
            if channel == pin_close and GPIO.input(pin_close):
               if auto_mlight_max <= 0: auto_mlight_max=255
               else: 
                  auto_mlight_max=auto_mlight_max-16 
                  if auto_mlight_max < 0: auto_mlight_max=0
            if channel == pin_open and GPIO.input(pin_open):
               if auto_mlight_max >= 255: auto_mlight_max=0
               else: 
                  auto_mlight_max=auto_mlight_max+16  
                  if auto_mlight_max > 255: auto_mlight_max=255

         # Licht Fading Zeit einstellen
         elif menu_mode == 17:
            if channel == pin_close and GPIO.input(pin_close):
               auto_mlight_ft=auto_mlight_ft-10
               if auto_mlight_ft < 0: auto_mlight_ft=0
            if channel == pin_open and GPIO.input(pin_open):
               auto_mlight_ft=auto_mlight_ft+10    

         # Licht-Sensor verzoegerung
         elif menu_mode == 18:
            if channel == pin_close and GPIO.input(pin_close):
               auto_lc=auto_lc-10
               if auto_lc < 0: auto_lc=0
            if channel == pin_open and GPIO.input(pin_open):
               auto_lc=auto_lc+10    

         # Motor Timeout
         elif menu_mode == 19:
            if channel == pin_close and GPIO.input(pin_close):
               motor_timeout=motor_timeout-1
               if motor_timeout < 0: motor_timeout=0
            if channel == pin_open and GPIO.input(pin_open):
               motor_timeout=motor_timeout+1     
 


      draw_lcd(adc.read(channel=ad_light),adc.read(channel=ad_temp))    

   # Motor-Steuerung ###############################################
   else:

      # Menu-Taste -> Ins Menu wechseln
      if channel == pin_menu and GPIO.input(pin_menu) and motor_status == 0:
         tmp_h=int(int(time.strftime("%H")))
         tmp_m=int(int(time.strftime("%M")))
         menu_mode=1
         draw_lcd(adc.read(channel=ad_light),adc.read(channel=ad_temp))

      # Taster fuer manuelles schliessen und manuelles steuern des Heizers
      elif channel == pin_close and GPIO.input(pin_close):
         if cur_page == 0  and motor_status == 0 and get_status() != 1:
            actionlogger.log("Manuelle Steuerung: Tuere schliessen")
            Set_motor(1)
         elif cur_page == 1 and (auto_heat_status > 0 or man_heater == True):
            man_heater = False
            auto_heat_status = 0
            actionlogger.log("Manuelle Steuerung: Heizer ausgeschaltet")
            GPIO.output(pin_heat,0)
         elif cur_page == 1 and (auto_heat_status == 0 and man_heater == False):
            man_heater = True
            auto_heat_status = 1
            actionlogger.log("Manuelle Steuerung: Heizer eingeschaltet")
            GPIO.output(pin_heat,1)

      # Taster fuer Stoppen des Antriebes bzw. umschalten der Page
      elif channel == pin_stop and GPIO.input(pin_stop):
         if motor_status != 0:
            actionlogger.log("Manuelle Steuerung: Tuereantrieb stoppen")
            Set_motor(0)
         elif cur_page == 0: cur_page=1
         elif cur_page == 1: cur_page=0

      # Taster fuer manuelles oeffnen oder manuelles steuern des Lichts
      elif channel == pin_open and GPIO.input(pin_open):
         if cur_page == 0 and motor_status == 0 and get_status() != -1:
            actionlogger.log("Manuelle Steuerung: Tuere oeffnen")
            Set_motor(-1)
         elif cur_page == 1 and ((auto_mlight_status > 0 and auto_mlight_status < 4) or (auto_alight_status > 0 and auto_alight_status < 4) or man_light == True):
            man_light=False
            if auto_mlight_status > 0: auto_mlight_status=4
            if auto_alight_status > 0: auto_alight_status=4
            auto_mlight_pwm = 0
            led.stop()
            actionlogger.log("Manuelle Steuerung: Licht ausgeschaltet")
         elif cur_page == 1 and (auto_mlight_status == 0 or auto_mlight_status == 4) and (auto_alight_status == 0 or auto_alight_status == 4) and man_light == False:
            man_light=True
            auto_mlight_pwm = auto_mlight_max
            led.start(led_steps[int(auto_mlight_pwm)])
            actionlogger.log("Manuelle Steuerung: Licht eingeschaltet")
            

      # Reed-Kontakt fuer Tuer-offen Endposition
      elif channel == pin_open_sens and get_status() == -1 and motor_status == -1:
         actionlogger.log("Automatik: Tuere geoeffnet, Motor stoppt")
         Set_motor(0)

      # Reed-Kontakt fuer Tuer-geschlossen Endposition
      elif channel == pin_close_sens and get_status() == 1 and motor_status == 1:
        actionlogger.log("Automatik: Tuere geschlossen, Motor stopp")
        Set_motor(0)


############################################################################
# Aufraeumarbeiten beim beenden ############################################
############################################################################
def exit_handler():
   lcd.lcd_clear()
   GPIO.cleanup()
   write_config()
   actionlogger.log("Chickensitter wird beendet, Bye :)")


############################################################################
# Interrupt-Setup ##########################################################
############################################################################
signal.signal(signal.SIGALRM, Timeout_handler)
GPIO.add_event_detect(pin_open,GPIO.RISING, callback = Taster_interrupt, bouncetime = 200)
GPIO.add_event_detect(pin_open_sens,GPIO.RISING,callback = Taster_interrupt, bouncetime = 50)
GPIO.add_event_detect(pin_close_sens,GPIO.RISING,callback = Taster_interrupt, bouncetime = 50)
GPIO.add_event_detect(pin_stop,GPIO.RISING, callback = Taster_interrupt, bouncetime = 50)
GPIO.add_event_detect(pin_close,GPIO.RISING, callback = Taster_interrupt, bouncetime = 200)
GPIO.add_event_detect(pin_menu,GPIO.RISING, callback = Taster_interrupt, bouncetime = 200)
atexit.register(exit_handler)


############################################################################
# Hauptschleife ############################################################
# -) Triggert die Tueroeffnung/Steuerung ueber Zeit/Licht automatik        #
# -) sorgt dafuer, dass das Programm nicht stoppt                          #
############################################################################
actionlogger.log("Der Chickensitter ist gestartet und steht zu Diensten!")
try:
   while True:

      # Aktuelle Uhrzeit in Stunde und Minute #####################################
      akt_h=int(int(time.strftime("%H")))
      akt_m=int(int(time.strftime("%M")))

      # Aktuelle Sensorwerte in Variablen uebernehmen #############################
      akt_lv=adc.read(channel=ad_light)
      akt_tv=adc.read(channel=ad_temp)

      # Automatische Funktionen sollen nur ausgefuehrt werden, wenn Menu nicht aktiviert
      if menu_mode == 0:


         # Nachfolgender Block steuert das automatische oeffnen. Nach Zeit und Licht
         if auto_open > 0:

            # Entprellung fuer Lichtsensor
            if akt_lv >= auto_open_lt:
               auto_open_lcc=auto_open_lcc+1
            else:
               auto_open_lcc=0
         
            # Trigger (Licht und Zeit) fuer Automatisches oeffnen pruefen und ggf.
            # Tuere oeffnen
            if auto_open_status == 0 and (auto_open_lcc >= auto_lc or auto_open != 2 or started == 0) and (akt_h > auto_open_h or (akt_h == auto_open_h and akt_m >= auto_open_m)):
               auto_open_status = 1
               if get_status() != -1 and started == 1:
                  Set_motor(-1)
                  actionlogger.log("Automatik: Tuere oeffnen")
       
            # Tueroeffner zuruecksetzen und somit auf naechsten automatischen oeffnungsvorgang vorbereiten
            # sobald uhrzeit wieder vor dem Limit liegt
            if auto_open_status == 1 and (akt_h < auto_open_h or (akt_h == auto_open_h and akt_m < auto_open_m)):
               auto_open_status = 0
               actionlogger.log("Automatik: Tueroeffner zurueckgesetzt")


         # Nachfolgender Block steuert das automatische schliessen. Nach Zeit und Licht
         if auto_close > 0:
         
            #Entprellung fuer Lichtsensor -> fuer Aktivierung
            if akt_lv < auto_close_lt:
               auto_close_lcc=auto_close_lcc+1
            else:
               auto_close_lcc=0

            #Entprellung fuer Lichtsensor -> fuer ruecksetzung
            if akt_lv >= auto_close_lt:
               auto_close_lccr=auto_close_lccr+1
            else:
               auto_close_lccr=0

            # Trigger (Licht und Zeit) fuer automatisches schliessen pruefen und ggf.
            # Tuere schliessen
            if auto_close_status == 0 and ((akt_h > auto_close_h or (akt_h == auto_close_h and akt_m >= auto_close_m)) or (auto_close_lcc >= auto_lc and auto_close >= 2)):
               auto_close_status = 1
               if get_status() != 1 and started == 1:
                  Set_motor(1)
                  actionlogger.log("Automatik: Tuere schliessen")
         
            # Tuerschliessen zuruecksetzen und somit auf naechste automatische schliessung vorbereiten
            # sobald uhrzeit und lichtwert vor/nach den Grenzwerten liegen
            if auto_close_status == 1 and (akt_h < auto_close_h or (akt_h == auto_close_h and akt_m < auto_close_m)) and (auto_close_lccr >= auto_lc or auto_close !=2 ):
               auto_close_status = 0
               actionlogger.log("Automatik: Tuerschliesser zurueckgesetzt")
      

         # Nachfolgender Block steuert den automatischen Temperaturgesteuerten Heizer
         if auto_heat == 1 and auto_heat_toff == 0:

            #Entprellung fuer Temperatursensor -> fuer Aktvierung
            if akt_tv < auto_heat_tt and auto_heat_status == 0:
               auto_heat_tcc=auto_heat_tcc+1
            else:
               auto_heat_tcc=0 

            #Entprellung fuer Temperatursensor -> fuer Reset
            if akt_tv >= auto_heat_tt and auto_heat_status == 1:
               auto_heat_tccr=auto_heat_tccr+1
            else:
               auto_heat_tccr=0

            #Heizer aktvieren, wenn Schwellenwert lange genug erreicht und aktuell deaktiviert
            if auto_heat_status == 0 and auto_heat_tcc >= auto_heat_tc:
               auto_heat_status=1
               actionlogger.log("Automatik: Heizer eingeschaltet")
               GPIO.output(pin_heat,1)

            #Heizer deaktivieren, wenn Schwellenwert lange genug nicht mehr erreicht und aktuell aktiviert
            if auto_heat_status == 1 and auto_heat_tccr >= auto_heat_tc and man_heater == False:
               auto_heat_status=0
               actionlogger.log("Automatik: Heizer ausgeschaltet")
               GPIO.output(pin_heat,0)

         
         # Nachfolgender Block steuert das automatische Morgenlicht
         if auto_mlight == 1 and man_light == False:

            #Entprelling fuer Lichtsensor -> fuer Ausschalten/Einschaltabfrage
            if akt_lv >= auto_mlight_lt:
               auto_mlight_lcc=auto_mlight_lcc+1
            else:
               auto_mlight_lcc=0

            #Entprellung fuer Lichtsensor -> fuer Reset
            if akt_lv < auto_mlight_lt and auto_mlight_status > 0:
               auto_mlight_lccr=auto_mlight_lccr+1
            else:
               auto_mlight_lccr=0
            

            if auto_mlight_status == 0 and akt_h < 12 and (akt_h > auto_mlight_h or (akt_h == auto_mlight_h and akt_m >= auto_mlight_m)):
               if (auto_mlight_lcc <= auto_lc and started == 1) or (akt_lv < auto_mlight_lt and started == 0):
                  auto_mlight_status=1
                  auto_mlight_sp=math.ceil(auto_mlight_max/(auto_mlight_ft/sleeptime))
                  auto_mlight_pwm=0
                  led.start(0)
                  actionlogger.log("Automatik: Morgen-Licht wird eingeschalten")
               else:
                  auto_mlight_status=4
                  actionlogger.log("Automatik: bereits zu hell fuer Morgen-Licht")
            elif auto_mlight_status == 1:
               auto_mlight_pwm=auto_mlight_pwm+auto_mlight_sp
               if auto_mlight_pwm >= auto_mlight_max:
                  auto_mlight_pwm=auto_mlight_max
                  auto_mlight_status=2
                  actionlogger.log("Automatik: Morgen-Licht hat hoechste Helligkeit erreicht")
               led.ChangeDutyCycle(led_steps[int(auto_mlight_pwm)])
            elif auto_mlight_status == 3:
               auto_mlight_pwm=auto_mlight_pwm-auto_mlight_sp
               if auto_mlight_pwm <= 0:
                  auto_mlight_pwm = 0
                  auto_mlight_status=4
                  led.stop()
                  actionlogger.log("Automatik: Morgen-Licht ist ausgeschaltet")
               else:
                  led.ChangeDutyCycle(led_steps[int(auto_mlight_pwm)])
            
            if auto_mlight_status > 0 and auto_mlight_status < 3 and auto_mlight_lcc >= auto_lc:
               auto_mlight_status = 3
               auto_mlight_sp=math.ceil(auto_mlight_max/(auto_mlight_ft/sleeptime))
               actionlogger.log("Automatik: Morgen-Licht wird ausgeschalten")

           
            if auto_mlight_status == 4 and akt_h < auto_mlight_h and auto_mlight_lccr >= auto_lc:
               auto_mlight_status=0
               actionlogger.log("Automatik: Morgen-Licht zurueckgesetzt")


         # Nachfolgender Block steuert das automatische Abendlicht
         if auto_alight == 1 and man_light == False:
            
            #Entprellung fuer Lichtsensor -> fuer Einschaltabfrage
            if akt_lv < auto_alight_lt:
               auto_alight_lcc=auto_alight_lcc+1
            else:
               auto_alight_lcc=0

            #Entprellung fuer Lichtsensor -> fuer Reset
            if akt_lv >= auto_alight_lt and auto_alight_status > 0:
               auto_alight_lccr=auto_alight_lccr+1
            else:
               auto_alight_lccr=0

           
            if auto_alight_status == 0 and auto_alight_lcc >= auto_lc:
               if akt_h > auto_alight_h or (akt_h == auto_alight_h and akt_m >= auto_alight_m):
                  auto_alight_status=4
                  actionlogger.log("Automatik: bereits zu spaet fuer Abend-Licht")
               else:
                  auto_alight_status=1
                  auto_mlight_sp=math.ceil(auto_mlight_max/(auto_mlight_ft/sleeptime))
                  auto_mlight_pwm=0
                  led.start(0)
                  actionlogger.log("Automatik: Abend-Licht wird eingeschalten")
            elif auto_alight_status == 1:
               auto_mlight_pwm=auto_mlight_pwm+auto_mlight_sp
               if auto_mlight_pwm >= auto_mlight_max:
                  auto_mlight_pwm=auto_mlight_max
                  auto_alight_status=2
                  actionlogger.log("Automatik: Abend-Licht hat hoechste Helligkeit erreicht")
               led.ChangeDutyCycle(led_steps[int(auto_mlight_pwm)])
            elif auto_alight_status == 3:
               auto_mlight_pwm=auto_mlight_pwm-auto_mlight_sp
               if auto_mlight_pwm <= 0:
                  auto_mlight_pwm = 0
                  auto_alight_status=4
                  led.stop()
                  actionlogger.log("Automatik: Abend-Licht ist ausgeschaltet")
               else:
                  led.ChangeDutyCycle(led_steps[int(auto_mlight_pwm)])

            if auto_alight_status > 0 and auto_alight_status < 3 and (akt_h > auto_alight_h or (akt_h == auto_alight_h and akt_m >= auto_alight_m)):
               auto_alight_status = 3
               auto_mlight_sp=math.ceil(auto_mlight_max/(auto_mlight_ft/sleeptime))
               actionlogger.log("Automatik: Abend-Licht wird ausgeschalten")

            if auto_alight_status == 4 and akt_h < auto_alight_h and auto_alight_lccr >= auto_lc:
               auto_alight_status=0
               actionlogger.log("Automatik: Abend-Licht zurueckgesetzt")
                  

      
         # LCD Aktualisieren
         if menu_mode >= 0: draw_lcd(akt_lv,akt_tv)

         
         # Start-Flag setzen
         if started == 0: started = 1

     
      #LCD Beleuchtung Restzeit reduzieren ggf. Beleuchtung abschalten
      if lcd_led_cur > 0:
         lcd_led_cur=lcd_led_cur-1
      elif lcd_led_cur == 0:
        lcd_led_cur=-1
        GPIO.output(pin_lcd_led,0)      

      # Counter fuer LCD-Reset reduzieren
      if lcd_reset_cur > 0: lcd_reset_cur=lcd_reset_cur-1

      # Sensorwerte loggen
      senslogger.log(akt_lv,akt_tv)

      time.sleep(sleeptime)

# Programm beenden #########################################################   
except KeyboardInterrupt:
   exit=1



